<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "check.class.php" );
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "action_in_margin.class.php" );

abstract class AbstractComponent {
  /*
   * Return the unique id for this component. The unique
   * id must be a triple: componentname.classname.identifier
   * Ids must be [A-Za-z0-9_-.]+
   */
  abstract public function id();
  /*
   * Return the unique name for this component.
   */
  public function component() {
    preg_match('/([^\.]+)/', $this->id(), $result);
    return $result[1];
  }
  /*
   * Return the title for this component.
   */
  abstract public function title();
  /*
   * The sort order for this component (low numbers are higher on the page).
   * Typically 1..100 but really just an integer.
   */
  public function sort_order() {
  	return 60;
  }
  /*
   * perform any checks or other activity that
   * must be done on a daily or periodic basis.
   */
  public function daily_activity() {}
  /*
   * Generate the full HTML for this component. The full
   * HTML includes the enclosing div with unique id and
   * the title.
   */
  public function outer_html($html = "") {
   ?>
<!-- std component -->
<div class="component">
  <div class="title"><?= $this->title() ?></div>
  <p><div id="<?= $this->id() ?>">
<?php
      if( $html ) {
	echo $html;
      } else {
	$this->inner_html();
      }
?>
  </div></p>
</div><!-- component -->
<?php
  }
  /*
   * Generate the inner HTML for this component.
   * Does not include the title.
   */
  abstract public function inner_html();

  public function revert_action() {
    $this->inner_html();
  }

  public function get_explore_url_for_action_named($name) {
    $component = $this->component();
    $class = get_class($this);
    $args = "component=$component&class=$class&action=$name";
    return "../tests/explore.php?$args";
  }

// html generators

  /*
   * Action link. Parameter is either:
   * 1. action name (old api, currently more common)
   * 2. margin_action object or subclass (new api, currently less common)
   */
  public function margin_action_html($name1,$name2 = null) {
  	$obj1 = $name1;
  	if( !($obj1 instanceof action_in_margin) ) {
  		$obj1 = new ajax_action_in_margin( $name1, $this );
  	}
	$obj2 = $name2;
        if( $obj2 != null && !($obj2 instanceof margin_action) ) {
                $obj2 = new ajax_action_in_margin( $name2, $this );
        }
	?>
	<div style="float:right; text-align: right;">
	<?php
	$obj1->action_html();
	if( $obj2 != null ) {
		?><br><?php $obj2->action_html(); ?><?php
	}
	?></div>
	<?php
  }

  public function explore_action_html($name) {
    ?>
      <a href=<?= $this->get_explore_url_for_action_named($name) ?> target=explore>explore</a>
    <?php
  }

  public function submit_action_html($name) {
    ?>
      <form action=dispatch.php method=post onSubmit="<?= $this->context->getSubmitLink($name, $this) ?>">
    <?php
  }

  public function html_submit_button($action) {
      echo "<input type=submit value=\"$action\">";
      $this->explore_action_html($action);
  }

  private $afrec = null;
  function af() { // action_function => outputs the action to the right, but only once
  	if( $this->afrec ){
  		$this->margin_action_html($this->afrec);
  		$this->afrec = null;
  	}
  }
  /*
   * Pass either:
   * 1. array of fields
   * 2. array of fields, action name
   * 3. array of (array of fields), array of action names
   */
  public function html_show_fields ($fields1, $action1 = null) {
  	$shownany = false;
  	$recs = array( array( $fields1, $action1 ) );
  	if( is_array($action1)) {
  		$recs = array();
  		for( $i = 0; $i < count($action1); $i++ ) {
  			$recs[]= array( $fields1[$i], $action1[$i] );
  		}
  	}
    ?>
      <table border="0" width="100%">
      <?php
      foreach( $recs as $rec ) {
	  	if( $rec[1] )
			$this->afrec = $rec[1];
        foreach ($rec[0] as $field) {
          $value = '';
          $value = "\$value = \$this->{$field['key']};";
          eval($value);
          if ($value != "") {
          	$shownany = true;
			if( !isset($field['type']) ) $field['type'] = "default";
			switch( $field['type'] ) {
		      case 'select':
		      	$translatedvalue = "???";
				foreach( $field['options'] as $option => $optionlabel) {
					if( ('' . $option) == ('' . $value) ) {
						$translatedvalue = $optionlabel;
						break;
					}
				}
          		?>
            	<tr><td align=right valign=top><?= $field['label'] ?>:</td>
            	<td>&nbsp;<td valign=top><?php $this->af(); ?><?= $translatedvalue ?>
            	<?php
		  	  break;
		  	  case 'image':
          		?>
            	<tr><td align=right valign=top><?= $field['label'] ?>:</td>
            	<td>&nbsp;<td valign=top><?php $this->af(); ?><img src="<?= $value ?>"/>
            	<?php
		  	  break;
		  	  case 'hidden':
		  	  	?>
		  	  	<input type='hidden' name='<?= $field['key'] ?>' value='<?= $value ?>' /> <?php
		  	  	break;
			  default:
			  	if( $field['label'] != '' ) {
	          		?>
	            	<tr><td align=right valign=top><?= $field['label'] ?>:</td>
	            <?php } else { ?>
	            	<tr><td align=right valign=top>&nbsp;</td>
	            <?php } ?>
            	<td>&nbsp;<td valign=top width=100%><?php $this->af(); ?><?= $value ?>
            	<?php
              break;
			}
			?></td></tr><?php
          }
        }
      }
      if( !$shownany ) {
        	?><tr><td><?php $this->af(); ?><font color=gray><em>[no data]</em></font></td></tr><?php
      }
      ?>
      </table>
   <?php
   }

  public function html_edit_fields (&$fields, $action) {
	$this->submit_action_html($action);
    ?>
      <table border="0">
      <?php
        foreach ($fields as $field) {
			$this->html_one_edit_field( $field );
		}
        ?>
      <tr><td><td><td>
      <? $this->html_submit_button($action); ?>
      </table></form>
    <?php
  }
  private function startrow( $field ) {
  		?>
          <tr><td valign=top align=right valign=top><?= $field['label'] ?>:
		<?php
  }
  public function html_one_edit_field( $field ) {
   	  if( !isset($field['type']) ) $field['type'] = "default";
      $value = "\$value = \$this->{$field['key']};";
 	  eval($value);
	  
	  switch( $field['type'] ) {
	    case 'label':
	    	$this->startrow($field);
            ?><td>&nbsp;</td><td valign=top><?= $value ?><input type="hidden" name="<?=$field['key']?>" value="<?= $value ?>"><?php
			break;
	    case 'textarea':
	    	$textarea_style = '';
			if (isset($value)) {
				$shown_value = $value;
			} elseif (isset($field['explain'])) {
				$textarea_style = 'style="color: grey;" onfocus="this.setAttribute(\'style\',\'color: black\'); this.value=\'\'; this.setAttribute(\'onfocus\',\'\');"';
				$shown_value = $field['explain'];
			} else {
				$shown_value = '';
			}
			$this->startrow($field);
			?>
			<td>&nbsp;</td><td valign=top><textarea name=<?=$field['key']?> rows=10 cols=33 wrap=virtual <?=$textarea_style?>><?= htmlspecialchars($shown_value) ?></textarea>
			<?php
   			break;
	    case 'select':
	    	if( count($field['options']) == 0 ) break;
	    	if( count($field['options']) == 1 ) {
	    		$this->startrow($field);
				foreach( $field['options'] as $option => $optionlabel) {
					$valuelabel = $optionlabel;
					$value = $option;
				}
	    		?><td>&nbsp;</td><td valign=top><?= $valuelabel ?>
	    		<input type="hidden" name="<?= $field['key']?>" value="<?= $value ?>">
	    		<?php
	    		if( $this->context->testmode ) $_POST[$field['key']] = $value;
				break;
	    	} else {
				$this->startrow($field);
		    	?>
				<td>&nbsp;</td><td valign=top><select size="1" name="<?= $field['key']?>">
				<?php
				$anychecked = false;
				foreach( $field['options'] as $option => $optionlabel) {
					$anychecked |= (('' . $option) == ('' . $value));
				}
				if( !$anychecked ) {
					?><option selected value=""> </option><?php
				}
				foreach( $field['options'] as $option => $optionlabel) {
					$checked = (('' . $option) == ('' . $value)) ? ' selected' : '';
					?><option <?= $checked ?> value="<?= $option ?>"><?= $optionlabel ?></option><?php
				}
		  		?>
		  		</select>
		  		<?php
			}
			break;
	  case 'checkbox':
			$this->startrow($field);
	    	?>
            <td>&nbsp;</td><td valign=top><?php
            $optArray = $field['options'];
            foreach( $optArray as $key => $element) {         	
          		$value = $this->{$key};	  
          		$checked = $value ? ' checked' : '';
          	?><input type=checkbox name=<?=$key?> value=<?=$value?> <?=$checked?>><?= $key ?><!-- -->&nbsp;&nbsp;<?php
          }
	      break;
	 
	    case 'radio':
			$this->startrow($field);
	    	?>
          <td>&nbsp;</td><td valign=top><?php
          foreach( $field['options'] as $option => $optionlabel) {
          	$checked = (('' . $option) == ('' . $value)) ? ' checked' : '';
          	?><input type=radio name=<?=$field['key']?> value="<?= $option ?>" <?= $checked ?>><?= $optionlabel ?><!-- -->&nbsp;&nbsp;<?php
          }
	      break;
	    case 'range':
			$this->startrow($field);
	    	?>
          <td>&nbsp;</td><td valign=top><table width=\"100%\" cellpadding=4><tr bgcolor="lightgray"><?php
          foreach( $field['options'] as $option => $optionlabel) {
          	$checked = (('' . $option) == ('' . $value)) ? ' checked' : '';
          	?><td align=center><input type=radio name=<?=$field['key']?> value="<?= $option ?>" <?= $checked ?>><!-- --><?php
          }
          ?><tr><?php
          foreach( $field['options'] as $option => $optionlabel) {
          	$checked = (('' . $option) == ('' . $value)) ? ' checked' : '';
          	?><td width=\"20%\" align=center><?= $optionlabel ?><?php
          }
          ?></table><?php
	      break;
       case 'hidden':
	  	  	?>
	  	  	<input type='hidden' name='<?= $field['key'] ?>' value='<?= $value ?>' /> <?php
	  	  	break;
	    default:
			$this->startrow($field);
	    	?>
          <td>&nbsp;<td valign=top><input type=text name=<?=$field['key']?> size=25 value="<?= htmlspecialchars($value) ?>">
		  <?php
	    break;
	  }
?>
          <?
          if (array_key_exists('error', $field)) echo "<br><font color=red>{$field['error']}</font>";
  }
  public function html_save_fields (&$fields) {
    $errors = 0;
    $record_changeds = array_key_exists('changed',get_object_vars($this));
    foreach ($fields as $row => $field) {
		$errors += $this->html_save_one_field( $row, $fields, $record_changeds );
    }
    return $errors;
  }
  public function html_save_one_field( $row, &$fields, $record_changeds ) {
  	  $errors = 0;
  	  $field = $fields[$row];
      $key = $field['key'];
      if (array_key_exists($key, $_POST)) {
        $value = $_POST[$key];
        if( get_magic_quotes_gpc() ) $value = stripslashes($value);
        if (array_key_exists('explain', $field)) {
        	if ($field['explain'] == $value) {
        		$value = '';
     		}
        }
        if (array_key_exists('check', $field)) {
          $check_spec = $field['check'];
        } elseif( isset($field['type']) && $field['type'] == 'select') {
		  if( !array_key_exists( $value, $field['options']) ) {
			$fields[$row]['error'] = 'Select one';
		  }
        } else {
          $check_spec = '4000';
        }
		$check = check_factory($check_spec);
		$fields[$row]['error'] = $error = $check->error($value);
		if ($error) $errors++;
        $oldvalue = $this->$key;
        if( $oldvalue != $value) {
      	  $this->$key = $value;
	  	  if( $record_changeds ) {
	      	  if( array_search( $key, $this->changed ) === false )
    	  	   	  $this->changed[] = $key;
      	  }
        }
      } else if( isset($field['type'])
      	&& ($field['type'] == 'radio' || $field['type'] == 'range') ) {
      	$fields[$row]['error'] = 'Must choose one';
      	$errors++;
      }
      return $errors;
  }
  public function edit_action() {
    $this->edit_html();
  }

  public function edit_html() {
    $this->margin_action_html('revert');
    $this->html_edit_fields($this->fields, 'save');
  }

  public function signal_create_component( $component ) {
      ?><!-- <?= $component->id() . "|" . $component->title() ?> --><?php
  }

  public function right_box_contribution() {
  	return null;
  }

  public function bottom_text_contribution() {
  	return null;
  }

  public $test_label;
  public function test_label() {
  	return $this->test_label;
  }

  public function anonymous_allowed() {
  	return false;
  }
}
?>
