<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
abstract class AbstractSchemaChecker {
	abstract public function check_and_modify( $context );

	public function check_and_modify_table( $databasename, $tablename, $schemas, $context ) {
		$dbh = $context->database( $databasename );

		$tablenamesuffix = $context->initmode;
		echo "$tablename$tablenamesuffix \n";
		$createfunction = "create_$tablename";
		if( $context->testmode
		 && $context->initmode === false ) {
			echo "..creating in memory <br>\n";
		 	$sql = "DROP TABLE IF EXISTS $tablename ";
		 	mysql_remember_query( $sql, $dbh );
		 	mysql_error_check();
		 	$sql = $this->$createfunction('');
		 	$translations = array(
				'/param(\d+) TEXT/' => 'param\1 varchar(1024)',
				'/\s[Tt][Ee][Xx][Tt]/' => ' varchar(1024)',
				'/blob/' => 'varbinary(8192)',
				'/ENGINE=([A-Za-z]+)/' => '',
				'/mediumtext/' => 'varchar(256)',
				'/tinytext/' => 'varchar(256)',
				);
		 	$a1 = array();
		 	$a2 = array();
		 	foreach( $translations as $k => $v ) {
		 		$a1[] = $k;
		 		$a2[] = $v;
		 	}
		 	$sql = preg_replace( $a1, $a2, $sql );
		 	$sql = $sql . " ENGINE=MEMORY SELECT * FROM $tablename" . '_prototype';
			$this->create_db( $sql,  $dbh );
			return true;
		} else {
			$result = mysql_remember_query( "DESCRIBE $tablename$tablenamesuffix", $dbh );
			if( strlen(mysql_error()) > 0 ) {
				if( $context->devmode
				 || $context->testmode
				 || $databasename == 'myfoundation' ) {
					echo "..does not exist, creating <br>\n";
					$this->create_db( $this->$createfunction($tablenamesuffix),  $dbh );
					$result = mysql_remember_query( "DESCRIBE $tablename$tablenamesuffix", $dbh );
					$str = $this->table_has_schema($result, $tablename . $tablenamesuffix, $schemas[count($schemas)], $dbh);
					if( $str === false )
						return true;
					else {
						echo $str;
						echo "..schema does not match after creation, error <br>\n";
						return false;
					}
				} else {
					echo "..does not exist, error <br>\n";
					return false;
				}
			}
			$lastmatch = 0;
			$laststr = '';
			for( $i = 1; $i <= count($schemas); $i++ ) {
				if( $i > 1 )
					$result = mysql_remember_query( "DESCRIBE $tablename$tablenamesuffix", $dbh );
				$str = $this->table_has_schema($result, $tablename . $tablenamesuffix, $schemas[$i], $dbh);
				if( $str === false )
					$lastmatch = $i;
				else
					$laststr = $str;
			}
			if( $lastmatch == 0 ) {
				if( $context->devmode
				 || $context->testmode ) {
		  			echo "..no matching schema, deleting and recreating <br>\n";
		  			mysql_remember_query( "DROP TABLE $tablename$tablenamesuffix", $dbh);
					mysql_error_check();
					$this->create_db( $this->$createfunction($tablenamesuffix),  $dbh );
					$result = mysql_remember_query( "DESCRIBE $tablename$tablenamesuffix", $dbh );
					$str = $this->table_has_schema($result, $tablename . $tablenamesuffix, $schemas[count($schemas)], $dbh);
					if( $str === false )
						return true;
					else {
						echo $str;
						echo "..schema does not match after creation, error <br>\n";
						return false;
					}
				} else {
					echo $laststr;
					echo "..no matching schema, error <br>\n";
					return false;
				}
			}
			if( $lastmatch == count($schemas) ) {
				echo "..correct <br>\n";
				return true;
			}
			if( $context->devmode
			 || $context->testmode
			 || $databasename == 'myfoundation' ) {
				echo "..old schema, updating <br>\n";
			 	for( $i = $lastmatch; $i < count($schemas); $i++ ) {
			 		$modifyfunction = 'modify_' . $tablename . '_' . $i . '_' . ($i+1);
			 		$this->$modifyfunction( $tablenamesuffix, $dbh );
			 	}
				$result = mysql_remember_query( "DESCRIBE $tablename$tablenamesuffix", $dbh );
				$str = $this->table_has_schema($result, $tablename . $tablenamesuffix, $schemas[count($schemas)], $dbh);
				if( $str === false )
					return true;
				else {
					echo $str;
					echo "..schema does not match after modification, error <br>\n";
					return false;
				}
			} else {
				echo "..old schema, error <br>\n";
				return false;
			}
		}
	}
	private function create_db( $sql, $dbh ) {
		mysql_remember_query( $sql, $dbh );
		mysql_error_check();
	}
	/* returns error string if error, null if schema matches */
	private function table_has_schema( $result, $tablename, $schemadescription, $dbh ) {
		$rtrn = '';
		$s1 = preg_split( "/[\r\n]+/", $schemadescription );
		while( substr($s1[0],0,4) != '+---' ) {
			array_shift( $s1 );
		}
		array_shift( $s1 ); // dashes
		array_shift( $s1 ); // field names
		array_shift( $s1 ); // dashes
		$schema = array();
		while( substr($s1[0],0,4) != '+---' ) {
			$words = preg_split( "/\|/", $s1[0] );
			array_shift( $s1 );
			$rec = array();
			$rec['field'] = trim($words[1]);
			$rec['type'] = trim($words[2]);
			if( preg_match( "/varchar\(\d+\)/", $rec['type'] )) $rec['type'] = 'text';
			$rec['null'] = trim($words[3]);
			$rec['key'] = trim($words[4]);
			$rec['default'] = trim($words[5]);
			$rec['extra'] = trim($words[6]);
			$rec['seen'] = false;
			$schema[$rec['field']] = $rec;
		}

		while( $row = mysql_fetch_assoc($result) ) {
			if( !array_key_exists($row['Field'], $schema) ) {
				$rtrn .= "..column " . $row['Field'] . " not in schema  <br>\n";
				$rtrn = false;
				continue;
			}
			$schema[$row['Field']]['seen'] = true;
			$rec = $schema[$row['Field']];
			if( preg_match( "/varchar\(\d+\)/", $row['Type'] )) $row['Type'] = 'text';
			if( $rec['type'] != $row['Type'] ) {
				$rtrn .= "..column " . $row['Field'] . " with different type (" . $rec['type'] . " vs " . $row['Type'] . ") <br>\n";
				continue;
			}
			$a = $rec['null'];
			$b = $row['Null'];
			if( $a == '' ) $a = 'NO';
			if( $b == '' ) $b = 'NO';
			if( $a != $b ) {
				$rtrn .= "..column " .  $row['Field']. " with different NULL (" . $rec['null'] . " vs " . $row['Null'] . ") <br>\n";
				continue;
			}
			if( $rec['key'] != $row['Key']) {
				if( $rec['key']) {
				  	$rtrn .= "..column " . $row['Field']. " with different keys (" . $rec['key'] . ") vs (" . $row['Key']. ") <br>\n";
					continue;
					// note that we allow the table to have more keys than the schema, but not fewer
				}
			}
			if( $rec['extra'] != $row['Extra']) {
			  	$rtrn .= "..column " . $row['Field']. " with different extra infomration (" . $rec['extra'] . " vs " . $row['Extra'] . ") <br>\n";
				continue;
			}
			// not able to check 'default' at this time
		}

		foreach( $schema as $rec ) {
			if( !$rec['seen']) {
				$rtrn .= "..column " . $rec['field'] . " not in table <br>\n";
				continue;
			}
		}
		if( $rtrn == '' ) return false;
		return $rtrn;
	}
}
?>
