<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
// This file should be sourced by all externally-facing scripts
// and external_entry() should be called appropriately.

// Setup some global variables used for calling between scripts
$cwd = getcwd();
$cwd = preg_replace( "/\\\\/", "/", $cwd );
$tmp = strrpos( $cwd, "/");
while( strlen($cwd) > 0 && substr( $cwd, $tmp ) != "/myfoundation-public") {
	$cwd = substr( $cwd, 0, $tmp );
	$tmp = strrpos( $cwd, "/");
}
$GLOBALS['ROOT_DIRECTORY'] = $cwd;
$GLOBALS['CLASSES_DIRECTORY'] = $cwd . "/classes/";
$GLOBALS['COMPONENTS_DIRECTORY'] = $cwd . "/components/";
$GLOBALS['WEB_HOME'] = substr($cwd, 0, strrpos($cwd, "/")); # Path below ROOT_DIRECTORY

// Set the timezone to default to Ottawa time
date_default_timezone_set("America/Montreal");

// Many portal-specific routines live in this file and it is needed globally
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "functions.php" );

// The global context variable
// Note that if you do not call external_entry() below you will not have a user
// assigned in the context.  If you need one you should assign it from
// $_SESSION['portaluser'] to the context, assuming a session has started.
$context = null;
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "context.class.php" );
require( $GLOBALS['CLASSES_DIRECTORY'] . "create-context.php" );

/*
 * Disable the system if the flag is set
 */
if( !isset($GLOBALS['SKIP_DISABLE_CHECK']) && file_exists( $GLOBALS['WEB_HOME'] . "/disable-portal.flag" ) ) {
	require_once( $GLOBALS['ROOT_DIRECTORY'] . "/portal/closed_message.php" );
	exit(0);
}

// Global Errror Handler Function
// This replicates the look/functionality of userErrorHandler from harness.php
// but doesn't implement the test-specific checks for global flags.
// NOTE:  This is not setup by default.  If you need to enable this,
// do something like: $old_error_handler = set_error_handler("globalDisplayErrorHandler");
// external_entry() sets this up if the context calls for it.
function globalDisplayErrorHandler($errno, $errmsg, $filename, $linenum, $vars) {
  $errortype = array (
    E_ERROR => 'Error',
    E_WARNING => 'Warning',
    E_PARSE => 'Parsing Error',
    E_NOTICE => 'Notice',
    E_CORE_ERROR => 'Core Error',
    E_CORE_WARNING => 'Core Warning',
    E_COMPILE_ERROR => 'Compile Error',
    E_COMPILE_WARNING => 'Compile Warning',
    E_USER_ERROR => 'User Error',
    E_USER_WARNING => 'User Warning',
    E_USER_NOTICE => 'User Notice',
    E_STRICT => 'Runtime Notice',
    E_RECOVERABLE_ERROR => 'Catchable Fatal Error');
    switch($errno) {
    	case E_NOTICE: // discard NOTICEs
    	case E_STRICT: // discard RUNTIME notices for deprecated usages
    		return;
    	default:
			echo "<p><table cellpadding=10 width=400 bgcolor=#ffcccc><tr><td><font size=+2>Trouble: </font>";
			echo "PHP $errortype[$errno]:<br>$errmsg<br>$filename ($linenum)";
			$mysql_error_func = 'mysql_error_check';
			if(function_exists($mysql_error_func)) {
				$mysql_error_func();
			}
			echo "</table></p>\n";
    }
}

function portal_header($header) {
?>
<html><head>
<META NAME="ROBOTS" CONTENT="NOINDEX,NOFOLLOW">
<link rel="stylesheet" type="text/css" href="http://www.eclipse.org/eclipse.org-common/themes/Phoenix/css/visual.css" media="screen" />
</head><body>
<table width="100%" border="0" cellpadding="0" cellspacing="0">
<tr id="header_row" style="background-image: url(../images/header_bg.gif);">
<td>
<a href="http://www.eclipse.org/"><img src="../images/header_logo.gif" width="163" height="68" border="0" alt="Eclipse Logo" class="logo" /></a>
</td>
<td align="right" style="color: white; font-family: verdana,arial,helvetica; font-size: 1.25em; font-style: italic;"><b><?= $header ?>&nbsp;</b></font> </td>
</tr>
</table>


<?php
}

// Every external facing script calls this function to setup the needed
// environment and optionally require a login if needed.
function external_entry( $dologin = true, $doheader = false) {
	global $context;
	/*
	 * Start the session and create the context
	 */
	session_start();
	if( isset( $_SESSION['portaluser'])) {
		$context->set("USER", $_SESSION['portaluser'] );
		$context->store_user_email();
	}

	// Setup a global error handler.  Since production blocks all logging and
	// dev systems may not all log errors, this allows the display of errors
	// or not as needed.  There could be errors that occur before this point
	// but this seems the best location for this.
	if($context->showerrors) {
		$old_error_handler = set_error_handler("globalDisplayErrorHandler");
	}

	/*
	 * Process a user logout
	 */
	if(isset($_GET['logout'])) {
		unset($_SESSION['portaluser']);
		session_destroy();
		Header("Location: portal.php");
	}
	/*
	 * If there is nobody logged in..
	 */
	if(!isset($_SESSION['portaluser']) && $dologin ) {
		/*
		 * If the user is here after filling out a login form..
		 */
		if(isset($_POST['user'])) {
			$loginsuccessful = false;
			// assume almost all devmode logins are successful, i.e., no password check
			// the special password 'error' causes a login error
			require_once("../classes/auth-foundationdb.class.php");
			$foundationdb_auth = new auth_foundationdb($context);

			if( $context->devmode ) {
				// As long as there is no password, let anyone in
				if($_POST['password'] != "") {
					// If there is a password, as long as it's not a member and the password
					// wasn't failed, then pass
					if($foundationdb_auth->authenticate($_POST['user'], $_POST['password']) == false) {
						$loginsuccessful = false;
					} else {
						$loginsuccessful = $_POST['password'] != 'error';
					}
				} else {
					$loginsuccessful = $_POST['password'] != 'error';
				}
			} else {
				// non-devmode, i.e., live production/staging, logins use LDAP
				require_once("../classes/auth-ldap.class.php");
				$ldap_auth = new auth_ldap($context);
				if($ldap_auth->authenticate($_POST['user'], $_POST['password'])) {
					$loginsuccessful = true;
				} elseif($foundationdb_auth->authenticate($_POST['user'], $_POST['password']) === true) {
					$loginsuccessful = true;
				}
			}
			if( !$loginsuccessful ) {
					$GLOBALS['MYFOUNDATION_LOGIN_ERROR'] = "Incorrect username or password";
					require_once( "../portal/login-page.php");
					exit(0);
			}
			/*
			 * When we get here, the user has successful authenticated
			 */
		    $_SESSION['portaluser'] = $_POST['user'];
		    header("Location: portal.php");
	    	//Note: the $context has pre-login user information, but that's ok because we are about to redirect via an exit
	    	exit(0);
		} else {
			$GLOBALS['MYFOUNDATION_LOGIN_ERROR'] = "";
			require_once( "../portal/login-page.php");
			exit(0);
		}
	} else {
		if ($doheader)
			portal_header(($doheader !== true) ? $doheader : "");
	}
}
?>