<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
// Global used for tracking SQL backtrace information
if(!isset($query_btrace)) {
	$query_btrace = array();
}

// Display a backtrace of all the SQL queries run in this session
function sql_backtrace () {
	global $query_btrace;
	global $context;
	if(!$context->showerrors) {
		return;
	}
	if(count($query_btrace) > 0) {
    	$row = 1;
    	echo "<p><table cellpadding=10 width=800 bgcolor=#ffcccc><tr><td>";
    	echo "<p><font size=\"+2\">Query Trace: </font> In ascending order from oldest to newest";
    	echo "<div style=\"font-family: courier;\">";
    	foreach ($query_btrace as $query) {
  			echo "&nbsp;&nbsp;&nbsp;&nbsp;<b>$row.) " .
  				$query{0} ." (" . $query{2} . ")rows:</b> " . $query{1} . "<br>\n";
  			$row++;
    	}
    	echo "</div>";
  		echo "</p>\n";
  		echo "</table></p>\n";
  	}
}

// MySQL wrapper function for keeping backtrace information
function mysql_remember_query ($statement, $dbh, $logstring = null) {
	global $query_btrace;
	global $context;

	$result = mysql_query($statement, $dbh);
	$rowcount = 0;
	// Report on the number of rows affected by the query
	if(($result !== TRUE) && ($result !== FALSE)) {
		$rowcount = mysql_num_rows($result);
	} else {
		$rowcount = mysql_affected_rows($dbh);
		if ($context->testmode && $rowcount > $context->max_affected_rows) {
			trouble("Too many rows affected ($rowcount>$context->max_affected_rows) for query: $statement");
		}
	}


	if( $logstring )
		//this is used when inserting binary blobs so that the blob does not appear in the log
		$query_btrace[] = array($context->database_name_for_handle($dbh), $logstring, $rowcount);
	else
			$query_btrace[] = array($context->database_name_for_handle($dbh), $statement, $rowcount);

	return $result;
}

// Check if a MySQL error occurred and display it.  If $context->showerrors then an
// SQL backtrace is shown as well.
function mysql_error_check () {
  $error = mysql_error();
  if ($error) {
  	echo "<p><table cellpadding=10 width=400 bgcolor=#ffcccc><tr><td><font size=+2>SQL Trouble: </font>";
    echo "<font color=red>";
    echo htmlspecialchars ($error);
    echo "</font>\n";
    echo "</table></p>\n";
    sql_backtrace();
    exit();
  }
}

// All in one query function
function sql ($statement, $dbname) {
	global $context;
	$dbh = $context->database( $dbname );

	$statement = str_replace ('%NOW%', $context->now(), $statement);
	$statement = str_replace ('%USER%', $context->user(), $statement);

	$result = mysql_remember_query ($statement, $dbh);
	mysql_error_check();
	return $result;
}

function portal_sql	($statement) 	{ return sql ($statement, "myfoundation"); }
function foundation_sql($statement) { return sql ($statement, "foundation"); }
function eclipse_sql ($statement) 	{ return sql ($statement, "eclipse"); }
function bugzilla_sql ($statement) 	{ return sql ($statement, "bugzilla"); }
function ipzilla_sql ($statement) 	{ return sql ($statement, "ipzilla"); }
function asterisk_sql ($statement) 	{ return sql ($statement, "asterisk"); }

/*
 * Queue up an email to be as soon as possible.  Emails
 * are queued rather than sent directly from the business objects
 * for two reasons: (1) email is a user interface function and
 * thus should not be part of the business object just as the web
 * page renderer is not part of the business object, and (2) it
 * may take a while to gather all the data and generate the email.
 * (A while: a few seconds.) We don't want to delay the business
 * object methods.
 *
 * Parameters:
 *  $component = the component sending the email
 *  $classfile = must be "foo" for a file named "emails/foo.php"
 *               containing a class named "foo_Email" which
 *               extends AbstractEmail
 *  $paramN    = parameters passed to the email renderer
 */
function send_immediate_email( $context, $component, $classfile,
	$param1 = "", $param2 = "", $param3 = "", $param4 = "" ) {
  $fromaddr = get_user_email_address($context);
  $fromaddr = add_automated_phrase($fromaddr);
  portal_sql( "INSERT INTO immediate_emails ( component, class, fromaddr, param1, param2, param3, param4 ) VALUES ( '$component', '$classfile', '$fromaddr', '$param1', '$param2', '$param3', '$param4' )");
}

function send_immediate_email_from( $context, $fromaddr, $component,
	$classfile, $param1 = "", $param2 = "", $param3 = "", $param4 = "" ) {
  $fromaddr = add_automated_phrase($fromaddr);
  portal_sql( "INSERT INTO immediate_emails ( component, class, fromaddr, param1, param2, param3, param4 ) VALUES ( '$component', '$classfile', '$fromaddr', '$param1', '$param2', '$param3', '$param4' )");
}

function send_daily_email( $context, $fromaddr, $component, $classfile, $param1 = "", $param2 = "", $param3 = "", $param4 = "" ) {
  $fromaddr = add_automated_phrase($fromaddr);
  portal_sql( "INSERT INTO daily_emails ( component, class, fromaddr, param1, param2, param3, param4 ) VALUES ( '$component', '$classfile', '$fromaddr', '$param1', '$param2', '$param3', '$param4' )");
}

function add_automated_phrase($addr) {
	if( preg_match('/(.*)\((.*)\)/', $addr, $matches ) ) {
		return $matches[1] . " (portal on behalf of " . $matches[2] . ")";
	} else {
		preg_match('/(.*)\@.*/', $addr, $matches);
		return "$addr (portal on behalf of " . $matches[1] . ")";
	}
}

function get_user_email_address($context, $userid = false, $unknown_from_address = 'portal-noreply@eclipse.org') {
	if ($userid === false) {
		return $context->user_email() . ' (' . $context->user_email_name() . ')';
	} else {
		$user = $userid;
	}
	if ($user == 'nobody') {
		return $unknown_from_address;
	}
	$result = foundation_sql( "SELECT FName, LName, EMail from People where PersonID = '$user';");
	$row = mysql_fetch_assoc( $result );
        if (mysql_num_rows($result) == 0) {
		return $unknown_from_address;
	}
	return $row['EMail'] . ' (' . $row['FName'] . ' ' . $row['LName'] . ')';
}


/*
 * functions to find files and dirs within a dir and return their names
 * as a sorted array.
 */

function dirs ($path) {
  if ($dh = opendir($path)) {
    while (($name = readdir($dh)) !== false) {
      if ($name[0] != '.' && $name != 'CVS' && is_dir("$path/$name")) {
        $names[] = $name;
      }
    }
  }
  closedir($dh);
  sort ($names);
  return $names;
}

function files ($path) {
  $names = array();
  if ($dh = opendir($path)) {
    while (($name = readdir($dh)) !== false) {
      if ($name[0] != '.' && is_file("$path/$name")) {
        $names[] = $name;
      }
    }
  }
  closedir($dh);
  sort ($names);
  return $names;
}

function elapsed ($seconds) {
  $minutes = round($seconds/60); if ($minutes < 2) return "$seconds seconds";
  $hours = round($minutes/60.5); if ($hours < 2) return "$minutes minutes";
  $days = round($hours/24); if ($days < 2) return "$hours hours";
  $weeks = round($days/7); if ($weeks < 2) return "$days days";
  $months = round($days/30.5); if ($months < 2) return "$weeks weeks";
  $years = round($days/365.5); if ($years < 2) return "$months months";
  $decades = round($years/10); if ($decades < 2) return "$years years";
  return "$decades decades";
}

/*
 * Live Email Delivery - don't call these from tests
 */
function deliver_email ($from, $to, $cc, $subject, $body, $filename, $context) {
  require_once( $GLOBALS['ROOT_DIRECTORY'] . "/pear/Mail.php");
  echo "send mail to $to\n";
  $headers['From']    = $from;
  $headers['To']      = $to;
  $headers['Cc']      = $cc;
  $headers['Subject'] = $subject;
  if( isset($cc) && $cc != null ) $headers['Cc'] = $cc;

  $params['host'] = $context->get('email-smtp-server');
  $params['port'] = $context->get('email-smtp-port');

  // Put this in a variable so we don't have to load all of PEAR every
  // single time we use functions.php.  This way it is hidden from the
  // parser but will work when needed.  Allows the require to be moved
  // inside the function.
  $mailfactory = '$mailfactory = Mail::factory(\'smtp\', $params);';

  // Create the mail object using the Mail::factory method
  eval($mailfactory);
  $mail_object =& $mailfactory;

  $mail_object->send($to . "," . $cc, $headers, $body);
}
// Used by file locking functions
function file_lock_error($str) {
	echo date(DATE_RFC822, time()) . ": $str\n";
}
// Used by cron scripts
// Note there could be a race condition here when writing,
// but we're only running at most once a minute so not much worry
function check_process_lock($name, $context) { // Name will be used to name the process file
	$lockfile = str_replace("%CRON%", $name, $context->get("cron_lockfile"));
	if(strlen($lockfile) < 1) {
		file_lock_error("ERROR: Bad filename: blank.\n");
		return false;
	}
	if(file_exists($lockfile)) {
		if(is_readable($lockfile)) {
			if(date("i", time()) % 5 != 0) {
				// Only send emails on the five minute mark regardless of how often we run
				return false;
			}
			file_lock_error("ERROR: lock file (" . $lockfile . ") exists and is in use.\n");
				$mail_method = $context->get( "email-delivery-function" );
				if(function_exists($mail_method)) {
					$mail_method (
						$context->get("webmaster_email"), // FROM
						$context->get("webmaster_email"), // TO
						'', 							  // CC
						"MyFoundation Portal Cron Error", // SUBJ
						"A process lock file already existed when $name cron tried to process ". // BODY
							"emails.  Please verify that the process is not locked.",
						"", // Unused
						$context // Context
					);
				} else {
					file_lock_error("ERROR: Invalid mail method defined ($mail_method) or correct file not sourced due to an error\n");
				}
			return false;
		} else {
			file_lock_error("ERROR: lock file exists and is not readable.  Permissions error?\n");
			return false;
		}
	}

	// Write this process identifier into the file
	if(!file_put_contents($lockfile, "lock")) {
		file_lock_error("ERROR: Unable to write to lock file: $lockfile\n");
		return false;
	}

	return true;
}

// Used from cron scripts
function remove_process_lock ($name = "portal", $context) {
	$lockfile = str_replace("%CRON%", $name, $context->get("cron_lockfile"));
	if(strlen($lockfile) < 1) {
		file_lock_error("ERROR: Bad filename: blank.\n");
		return false;
	}
	if(unlink($lockfile)) {
		return true;
	}
	file_lock_error("ERROR: Can't remove lock file!");
	return false;
}

function add_to_modlog( $tablename, $pk1, $pk2, $action, $context ) {
	$sql = "INSERT INTO SYS_ModLog (
						LogTable,
						PK1,
						PK2,
						LogAction,
						PersonID,
						ModDateTime)
					VALUES (
						'$tablename',
						'$pk1',
						'$pk2',
						'$action',
						'" . $context->user() . "',
						NOW()
					)";
	foundation_sql( $sql );
}

function user_has_role( $context, $relation) {
	$name = $context->user();
	$result = foundation_sql( "SELECT COUNT(*) FROM PeopleRelations " .
    		"WHERE PersonID = '$name' AND Relation = '$relation'");
    $row = mysql_fetch_array($result);
    return $row[0] > 0;
}

function user_has_emo_portal_role( $context ) {
    return user_has_role($context, 'PORTL');
}
function user_has_emo_meeting_role( $context ) {
    return user_has_role($context, 'MEETG');
}

function user_has_emo_legal_role( $context ) {
    return user_has_role($context, 'EL');
}

function user_has_webmaster_role( $context ) {
    return user_has_role($context, 'WEBM');
}

function user_has_emo_membership_role( $context ) {
    return user_has_role($context, 'EM');
}

function user_has_committer_role( $context, $personid=false, $project=false) {
	$name = $personid ? $personid : $context->user();
	$project_phrase = $project ? " AND ProjectID = '$project'" : '';

	$result = foundation_sql( "SELECT COUNT(*) FROM PeopleProjects " .
    		"WHERE PersonID = '$name' AND Relation = 'CM' AND InactiveDate IS NULL $project_phrase");
    $row = mysql_fetch_array($result);
    return $row[0] > 0;
}

function organization_name( $id, $context ) {
	$name = $context->user();
	$result = foundation_sql( "SELECT Name1, Name2 FROM Organizations " .
    		"WHERE OrganizationID = $id");
    $row = mysql_fetch_array($result);
    return $row[0] . ' ' . $row[1];
}

function person_exists( $PersonID, $context ) {
	$result = foundation_sql("SELECT count(1) FROM People AS found WHERE PersonID = '$PersonID'");
	$row = mysql_fetch_assoc($result);
	return $row['found'];
}

function array_of_it( $result ) {
	$rtrn = array();
	while( $row = mysql_fetch_assoc($result) ) {
		$rtrn[] = $row['it'];
	}
	return $rtrn;
}
?>
