<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "functions.php" );
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "context.class.php" );
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "check-database-schema.class.php" );

class InitializeDemoDatabase {
  public function initialize( $context ) {
    if( !($context->devmode || $context->testmode) ) {
?>
      <h1>Warning: Attempt made to initialize demo databases on production server.
This is a really bad idea as it would delete all the production data.
Disallowing the attempt and terminating the script.</h1>
<?php
      die(1);
    }

    $foundation_dbname = $context->database_name('foundation');
    if( ($foundation_dbname == 'eclipsefoundation') ) {
?>
<h2>Warning: the foundation database is named 'eclipsefoundation' 
which looks like a production system.  This script
will not makes any changes and will exit.</h2>
<?php
      die(1);
    }
 
    $myfoundation_dbname = $context->database_name('myfoundation');
    if( $myfoundation_dbname == 'myfoundation_prod' ) {
?>
<h2>Warning: the myfoundation database is named 'myfoundation_prod'
which looks like a production system.  This script
will not makes any changes and will exit.</h2>
<?php
      die(1);
    }
   	echo "<h3> Checking database schemas </h3>\n";
   	$dbname = $context->database_name( "myfoundation" );
    if( $context->devmode && !$context->testmode ) {
	    portal_sql( "DROP DATABASE IF EXISTS $dbname;");
	    portal_sql( "CREATE DATABASE IF NOT EXISTS $dbname;");
    }
    
    $checker = new CheckAndModifyDatabaseSchema();
    if( !$checker->check_and_modify( $context ) ) {
    	echo "<font color=red>Trouble: database 'check_and_modify' returned false </font><br>\n";
    	return;
    }
    
   	echo "<h3> Creating demo content </h3>\n";
    $dir = $context->get("components_directory");
    foreach( dirs($dir) as $component ) {
      if( file_exists( $dir . "$component/initialize-demo-database.php" ) ) {
		$suffix = $context->initmode;
      	echo "<b> Creating demo db content for $component </b><br>\n";
        include_once( $dir . "$component/initialize-demo-database.php" );
      }
    }
  }
}

?>
