<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "abstractschemachecker.class.php" );
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "functions.php" );
/*
 * For a complete, ready-to-copy, example of a component's schema checker,
 * please see the committer_vote component.
 */
class contact_address_SchemaChecker extends AbstractSchemaChecker {
  public function check_and_modify( $context ) {
	/* Don't touch the production database - only modify the databases on the dev machines */
	if( $context->devmode || $context->testmode ) {
		echo "[disabling foreign key constraint checking] <br>\n";
		foundation_sql("SET FOREIGN_KEY_CHECKS = 0");
	}

	$rtrn = $this->check_SYS_Countries( $context )
		 && $this->check_SYS_Relations( $context )
		 && $this->check_People( $context )
		 && $this->check_PeopleAddresses( $context )
		 && $this->check_PeopleProjects( $context )
		 && $this->check_PeopleRelations( $context )
		 && $this->check_Projects( $context );

	/* Don't touch the production database - only modify the databases on the dev machines */
	if( $context->devmode || $context->testmode ) {
		echo "[enabling foreign key constraint checking] <br>\n";
		foundation_sql("SET FOREIGN_KEY_CHECKS = 1");
	}
	return $rtrn;
  }

  public function check_People( $context ) {
	$schemas = array();
	$schemas[1] = "
+-------------------+---------------------+------+-----+---------+-------+
| Field             | Type                | Null | Key | Default | Extra |
+-------------------+---------------------+------+-----+---------+-------+
| PersonID          | varchar(20)         | NO   | PRI |         |       |
| FName             | varchar(100)        | NO   |     |         |       |
| LName             | varchar(100)        | NO   |     |         |       |
| Type              | char(2)             | NO   |     |         |       |
| IsMember          | tinyint(1) unsigned | NO   |     | 0       |       |
| EMail             | varchar(200)        | NO   |     |         |       |
| Phone             | varchar(40)         | YES  |     | NULL    |       |
| Fax               | varchar(40)         | YES  |     | NULL    |       |
| Mobile            | varchar(40)         | YES  |     | NULL    |       |
| Comments          | text                | YES  |     | NULL    |       |
| Provisioning      | text                | YES  |     | NULL    |       |
| IsUnixAcctCreated | tinyint(3) unsigned | NO   |     | 1       |       |
| MemberSince       | date                | YES  |     | NULL    |       |
| IssuesPending     | tinyint(1)          | YES  |     | 0       |       |
| MemberPasswd      | varchar(40)         | YES  |     | NULL    |       |
+-------------------+---------------------+------+-----+---------+-------+
";
	return $this->check_and_modify_table( 'foundation', 'People', $schemas, $context );
  }
  public function create_People($suffix ) {
	return
			  "CREATE TABLE People$suffix (
			  PersonID varchar(20) NOT NULL default '',
			  FName varchar(100) NOT NULL default '',
			  LName varchar(100) NOT NULL default '',
			  Type char(2) NOT NULL default '',
			  IsMember tinyint(1) unsigned NOT NULL default '0',
			  EMail varchar(200) NOT NULL default '',
			  Phone varchar(40) default NULL,
			  Fax varchar(40) default NULL,
			  Mobile varchar(40) default NULL,
			  Comments text,
			  Provisioning text,
			  IsUnixAcctCreated tinyint(3) unsigned NOT NULL default '1',
			  MemberSince date default NULL,
			  IssuesPending tinyint(1) default '0',
			  MemberPasswd varchar(40) default NULL,
			  PRIMARY KEY (PersonID)) ENGINE=InnoDB DEFAULT CHARSET=latin1"
		  ;
  }

  public function check_PeopleAddresses( $context ) {
		$schemas = array();
$schemas[1] = "
+-----------------+------------------+------+-----+---------+-------+
| Field           | Type             | Null | Key | Default | Extra |
+-----------------+------------------+------+-----+---------+-------+
| PersonID        | varchar(20)      | NO   | PRI |         |       |
| AddressID       | int(10) unsigned | NO   | PRI | 0       |       |
| Address1        | varchar(200)     | NO   |     |         |       |
| Address2        | varchar(200)     | NO   |     |         |       |
| Address3        | varchar(200)     | NO   |     |         |       |
| City            | varchar(100)     | NO   |     |         |       |
| ProvStateRegion | varchar(100)     | NO   |     |         |       |
| PostalCode      | varchar(100)     | NO   |     |         |       |
| CCode           | char(2)          | NO   | MUL |         |       |
+-----------------+------------------+------+-----+---------+-------+
";
	return $this->check_and_modify_table( 'foundation', 'PeopleAddresses', $schemas, $context );
  }
  public function create_PeopleAddresses( $suffix) {
		return "CREATE TABLE PeopleAddresses$suffix (
			  `PersonID` varchar(20) NOT NULL default '',
			  `AddressID` int(10) unsigned NOT NULL default '0',
			  `Address1` varchar(200) NOT NULL default '',
			  `Address2` varchar(200) NOT NULL default '',
			  `Address3` varchar(200) NOT NULL default '',
			  `City` varchar(100) NOT NULL default '',
			  `ProvStateRegion` varchar(100) NOT NULL default '',
			  `PostalCode` varchar(100) NOT NULL default '',
			  `CCode` char(2) NOT NULL default '',
			  PRIMARY KEY  (`PersonID`,`AddressID`),
			  KEY `FK_ccode` (`CCode`),
			  CONSTRAINT `FK_ccode` FOREIGN KEY (`CCode`) REFERENCES `SYS_Countries$suffix` (`CCode`) ON UPDATE CASCADE,
			  CONSTRAINT `FK_PersonID` FOREIGN KEY (`PersonID`) REFERENCES `People$suffix` (`PersonID`) ON DELETE CASCADE ON UPDATE CASCADE
			) ENGINE=InnoDB DEFAULT CHARSET=latin1";
  }

  public function check_PeopleProjects( $context ) {
		$schemas = array();
$schemas[1] = "
+--------------+--------------+------+-----+------------+-------+
| Field        | Type         | Null | Key | Default    | Extra |
+--------------+--------------+------+-----+------------+-------+
| PersonID     | varchar(20)  | NO   | PRI |            |       |
| ProjectID    | varchar(100) | NO   | PRI |            |       |
| Relation     | varchar(5)   | NO   | PRI |            |       |
| ActiveDate   | date         | NO   |     | 0000-00-00 |       |
| InactiveDate | date         | YES  |     | NULL       |       |
| EditBugs     | tinyint(1)   | NO   |     | 0          |       |
+--------------+--------------+------+-----+------------+-------+
";
	return $this->check_and_modify_table( 'foundation', 'PeopleProjects', $schemas, $context );
  }
  public function create_PeopleProjects($suffix ) {
		return "CREATE TABLE `PeopleProjects$suffix` (
			  `PersonID` varchar(20) NOT NULL default '',
			  `ProjectID` varchar(100) NOT NULL default '',
			  `Relation` varchar(5) NOT NULL default '',
			  `ActiveDate` date NOT NULL default '0000-00-00',
			  `InactiveDate` date default NULL,
			  `EditBugs` tinyint(1) NOT NULL default '0',
			  PRIMARY KEY  (`PersonID`,`ProjectID`,`Relation`),
			  KEY `FK_1` (`ProjectID`),
			  KEY `FK_3` (`Relation`),
			  CONSTRAINT `FK_1` FOREIGN KEY (`ProjectID`) REFERENCES `Projects$suffix` (`ProjectID`) ON UPDATE CASCADE,
			  CONSTRAINT `FK_2` FOREIGN KEY (`PersonID`) REFERENCES `People$suffix` (`PersonID`) ON DELETE CASCADE ON UPDATE CASCADE,
			  CONSTRAINT `FK_3` FOREIGN KEY (`Relation`) REFERENCES `SYS_Relations$suffix` (`Relation`) ON UPDATE CASCADE
			) ENGINE=InnoDB DEFAULT CHARSET=latin1";
  }

  public function check_PeopleRelations( $context ) {
		$schemas = array();
$schemas[1] = "
+-----------+-------------+------+-----+---------+-------+
| Field     | Type        | Null | Key | Default | Extra |
+-----------+-------------+------+-----+---------+-------+
| PersonID  | varchar(20) | NO   | PRI |         |       |
| Relation  | varchar(5)  | NO   | PRI |         |       |
| EntryDate | date        | YES  |     | NULL    |       |
+-----------+-------------+------+-----+---------+-------+
";
	return $this->check_and_modify_table( 'foundation', 'PeopleRelations', $schemas, $context );
  }
  public function create_PeopleRelations( $suffix) {
		return "CREATE TABLE `PeopleRelations$suffix` (
			  `PersonID` varchar(20) NOT NULL default '',
			  `Relation` varchar(5) NOT NULL default '',
			  `EntryDate` date default NULL,
			  PRIMARY KEY  (`PersonID`,`Relation`),
			  KEY `fk_2` (`Relation`),
			  CONSTRAINT `PeopleRelations_ibfk_1` FOREIGN KEY (`Relation`) REFERENCES `SYS_Relations$suffix` (`Relation`) ON DELETE CASCADE ON UPDATE CASCADE,
			  CONSTRAINT `PeopleRelations_ibfk_2` FOREIGN KEY (`PersonID`) REFERENCES `People$suffix` (`PersonID`) ON DELETE CASCADE ON UPDATE CASCADE
			) ENGINE=InnoDB DEFAULT CHARSET=latin1";
  }

  public function check_Projects( $context ) {
		$schemas = array();
$schemas[1] = "
+-----------------+---------------------+------+-----+------------+-------+
| Field           | Type                | Null | Key | Default    | Extra |
+-----------------+---------------------+------+-----+------------+-------+
| ProjectID       | varchar(100)        | NO   | PRI |            |       |
| Name            | varchar(200)        | NO   |     |            |       |
| Level           | tinyint(3) unsigned | NO   |     | 0          |       |
| ParentProjectID | varchar(100)        | NO   |     |            |       |
| Description     | text                | NO   |     |            |       |
| UrlDownload     | varchar(255)        | YES  |     |            |       |
| UrlIndex        | varchar(255)        | YES  |     |            |       |
| DateActive      | date                | NO   |     | 0000-00-00 |       |
| SortOrder       | int(10) unsigned    | YES  |     | 0          |       |
| IsActive        | tinyint(1)          | YES  |     | 1          |       |
| BugsName        | varchar(64)         | YES  |     | NULL       |       |
| ProjectPhase    | varchar(20)         | YES  |     | NULL       |       |
+-----------------+---------------------+------+-----+------------+-------+
";
	return $this->check_and_modify_table( 'foundation', 'Projects', $schemas, $context );
  }
  public function create_Projects( $suffix) {
		return "CREATE TABLE `Projects$suffix` (
			  `ProjectID` varchar(100) NOT NULL default '',
			  `Name` varchar(200) NOT NULL default '',
			  `Level` tinyint(3) unsigned NOT NULL default '0',
			  `ParentProjectID` varchar(100) NOT NULL default '',
			  `Description` text NOT NULL,
			  `UrlDownload` varchar(255) default '',
			  `UrlIndex` varchar(255) default '',
			  `DateActive` date NOT NULL default '0000-00-00',
			  `SortOrder` int(10) unsigned default '0',
			  `IsActive` tinyint(1) default '1',
			  `BugsName` varchar(64) default NULL,
			  `ProjectPhase` varchar(20) default NULL,
			  PRIMARY KEY  (`ProjectID`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1";
  }

  public function check_SYS_Countries( $context ) {
		$schemas = array();
$schemas[1] = "
+----------------+--------------+------+-----+---------+-------+
| Field          | Type         | Null | Key | Default | Extra |
+----------------+--------------+------+-----+---------+-------+
| CCode          | char(2)      | NO   | PRI |         |       |
| en_Description | varchar(200) | NO   |     |         |       |
| fr_Description | varchar(200) | YES  |     | NULL    |       |
+----------------+--------------+------+-----+---------+-------+
";
	return $this->check_and_modify_table( 'foundation', 'SYS_Countries', $schemas, $context );
  }
  public function create_SYS_Countries($suffix ) {
		return "CREATE TABLE `SYS_Countries$suffix` (
			  `CCode` char(2) NOT NULL default '',
			  `en_Description` varchar(200) NOT NULL default '',
			  `fr_Description` varchar(200) default NULL,
			  PRIMARY KEY  (`CCode`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1";
  }

  public function check_SYS_Relations( $context ) {
		$schemas = array();
$schemas[1] = "
+----------------+--------------+------+-----+---------+-------+
| Field          | Type         | Null | Key | Default | Extra |
+----------------+--------------+------+-----+---------+-------+
| Relation       | varchar(5)      | NO   | PRI |         |       |
| Description    | varchar(100) | NO   |     |         |       |
| IsActive       | tinyint(1)   | NO   |     | 0       |       |
| Type           | char(2)      | NO   |     | 0       |       |
+----------------+--------------+------+-----+---------+-------+
";
	return $this->check_and_modify_table( 'foundation', 'SYS_Relations', $schemas, $context );
  }
  public function create_SYS_Relations( $suffix) {
		return "CREATE TABLE `SYS_Relations$suffix` (
			  `Relation` varchar(5) NOT NULL default '',
			  `Description` varchar(100) NOT NULL default '',
			  `IsActive` tinyint(1) NOT NULL default '0',
			  `Type` char(2) NOT NULL default '',
			  PRIMARY KEY  (`Relation`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1";
  }
}
?>
