<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "abstractschemachecker.class.php" );
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "functions.php" );

class poll_SchemaChecker extends AbstractSchemaChecker {
  public function check_and_modify( $context ) {
  	return $this->check_and_modify_poll_question( $context )
  		&& $this->check_and_modify_poll_answer( $context );
  }


  private function check_and_modify_poll_question( $context ) { 
	$schemas = array();
	$schemas[1] = "
+-----------+----------+------+-----+---------+----------------+
| Field     | Type     | Null | Key | Default | Extra          |
+-----------+----------+------+-----+---------+----------------+
| pollid    | int(11)  | NO   | PRI | NULL    | auto_increment | 
| statement | text     | NO   |     |         |                | 
| submit    | datetime | YES  |     | NULL    |                | 
| post      | date     | YES  |     | NULL    |                | 
+-----------+----------+------+-----+---------+----------------+
";
	return $this->check_and_modify_table( 'myfoundation', 'poll_question', $schemas, $context );
  }

  public function create_poll_question($suffix) {
	return "CREATE TABLE `poll_question$suffix` (
      `pollid` int(11) NOT NULL auto_increment,
      `statement` text NOT NULL,
      `submit` datetime default NULL,
      `post` date default NULL,
      PRIMARY KEY  (`pollid`)
    )";
  }


  private function check_and_modify_poll_answer( $context ) {
	$schemas = array();
	$schemas[1] = "
+-------------+-------------+------+-----+---------+-------+
| Field       | Type        | Null | Key | Default | Extra |
+-------------+-------------+------+-----+---------+-------+
| pollid      | int(11)     | NO   | PRI | 0       |       | 
| PersonID    | varchar(20) | NO   | PRI |         |       | 
| endorsement | tinyint(1)  | NO   |     | 0       |       | 
| agreement   | int(11)     | YES  |     | NULL    |       | 
| comment     | text        | YES  |     | NULL    |       | 
+-------------+-------------+------+-----+---------+-------+
";
	return $this->check_and_modify_table( 'myfoundation', 'poll_answer', $schemas, $context );
  }

  public function create_poll_answer($suffix ) {
	return "CREATE TABLE `poll_answer$suffix` (
      `pollid` int(11) NOT NULL default '0',
      `PersonID` varchar(20) NOT NULL default '',
      `endorsement` tinyint(1) NOT NULL default 0,
      `agreement` int(11) default NULL,
      `comment` text,
      PRIMARY KEY  (`pollid`,`PersonID`)
    )";
  }
}
?>
