<?php
/*******************************************************************************
 * Copyright (c) 2006-2007 Eclipse Foundation
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Bjorn Freeman-Benson - initial API and implementation
 *    Ward Cunningham - initial API and implementation
 *    Karl Mathias - initial API and implementation
 *******************************************************************************/
require_once( $GLOBALS['CLASSES_DIRECTORY'] . "ifactory.class.php" );

class poll_Factory implements IFactory {

    public function getObjectsForContext( $context ) {
        if ( user_has_committer_role( $context) ) {
            return array (new poll ($context));
        }
        if ( false ) { 
        	// read-only version of poll
        	return array (new poll_watch ($context));
        }
        return array();
    }

    public function getObjectForId( $classname, $id, $context ) {
        if( $classname != "poll" && $classname != "poll_watch")
            return null;
        return new $classname( $context);
    }

	public function getActiveObjects( $context ) {
		$rtrn=array();
        $now = $context->now();
		$result = portal_sql ("SELECT pollid
                FROM poll_question
            WHERE post > $now - interval 1 week + interval 1 day
                AND post <= $now");
		if (mysql_num_rows($result) == 0) {
			$rtrn[] = new poll($context);
		}
		return $rtrn;
	}
}


require_once( $GLOBALS['CLASSES_DIRECTORY'] . "abstractcomponent.class.php" );

class poll extends AbstractComponent {

    public $context;
    public $pollid;
    public $statement;
    public $has_answered = false;
    public $has_endorsed = false;
    public $counts = array(0,0,0,0,0);
    public $bins = array('','','','','');

    public $poll_fields;
    public $agreement;
    public $endorsement;
    public $comment;
    public $input;




    public function poll ($context) {
        $this->context = $context;
        $this->init_question();
        $this->init_answer();
        $this->has_answered = isset($this->agreement);
        $this->has_endorsed = isset($this->endorsement);

        $this->poll_fields = array (
			array ('key'=>'agreement', 'label'=>'Answer', 'type'=>'range', 'check'=>'req',
				'options' => array( '0'=>'disagree', '1'=>'', '2'=>'neutral', '3'=>'', '4'=>'agree')),
			array ('key'=>'comment', 'label'=>'Comments', 'type'=>'textarea', 'explain'=>'add opinion or other information here')
    );
    }

    public function id() {
        return "poll.poll.1";
    }

    public function title() {
        return "Committer Question of the Week";
    }
	public function sort_order() {
		return 20;
	}
    public function daily_activity() {
    	$result = portal_sql ("SELECT q.pollid, COUNT(endorsement) as count FROM poll_answer as a, poll_question as q
    		WHERE a.pollid = q.pollid AND q.post IS NULL GROUP BY a.pollid ORDER BY count DESC, q.submit");
    	if (mysql_num_rows($result) == 0) {
	    	$result = portal_sql ("SELECT pollid FROM poll_question WHERE post IS NULL ORDER BY submit");
		}
    	if (mysql_num_rows($result) > 0) {
    		$row = mysql_fetch_array($result);
    		$pollid = $row['pollid'];
			$this->schedule($pollid);
    	}

    }

    public function schedule ($pollid) {
    	$date = $this->context->now() . " + interval 1 day";		// to be active tomorrow
    	$date = "date_sub($date, interval weekday($date) day)";		// but always start at begining of week
    	portal_sql ("UPDATE poll_question SET post = $date WHERE pollid = $pollid");

    }

    public function schedule_week ($pollid, $week) {
    	$date = $this->context->now() ;								// relative to now
    	$date = "date_sub($date, interval weekday($date) day)";		// but always start at begining of week
    	$date = "$date + interval $week week";						// active $week weeks later
    	portal_sql ("UPDATE poll_question SET post = $date WHERE pollid = $pollid");

    }

    public function init_question() {
        $now = $this->context->now();
        $result = portal_sql ("
            SELECT pollid, statement
                FROM poll_question
            WHERE post > $now - interval 1 week
                AND post <= $now");
        $row = mysql_fetch_array($result);
        $this->pollid = $row['pollid'];
        $this->statement = stripslashes($row['statement']);
    }

    public function init_counts() {
    	$number = 1;
    	$this->counts = array(0,0,0,0,0);
    	$this->bins = array('','','','','');
        $result = portal_sql("SELECT agreement, comment, PersonID FROM poll_answer WHERE pollid = $this->pollid");
        while($row = mysql_fetch_array($result)) {
            $this->counts[$row['agreement']]++;
            $comment = stripslashes($row['comment']);
            $width = '200px';
	    $mine = $row['PersonID'] == $this->context->user();
            if( $comment == '' ) {
            	$item = $mine ? '<font color=green> o </font>' : ' o ';
            } else {
            	if( strlen($comment) < 40 ) $comment .= '<p>&nbsp;';
            	$item =  " <span onmouseover=\"over(this,'comment$number')\" onmouseout=\"out('comment$number')\">";
		$item .= $mine ? '<font color=green> O </font>' : ' O ';
		$item .= "</span>
			<div class=\"tip\" id=\"comment$number\">
			<TABLE WIDTH=$width BORDER=0 CELLPADDING=0 CELLSPACING=0>
			<TR><TD class=\"balloon1\"></TD><TD class=\"balloon2\"></TD><TD class=\"balloon3\"></TD></TR><TR><TD class=\"balloon4\"></TD>
			<TD class=\"balloon5\" id=\"comment${number}inner\">$comment</TD>
			<TD class=\"balloon6\"></TD></TR><TR><TD class=\"balloon7\"></td><TD class=\"balloon8\"></td><TD class=\"balloon9\"></td></tr></TABLE>
			</div>
            ";
            	$number++;
            }
            $this->bins[$row['agreement']] .= $item;
        }
    }

    public function init_answer() {
    	if ($this->pollid == '') return;
        $result = portal_sql("SELECT agreement, endorsement, comment FROM poll_answer WHERE pollid = $this->pollid AND PersonID = '%USER%'");
        $row = mysql_fetch_array($result);
        if($row) {
        	$this->agreement = $row['agreement'];
        	$this->comment = stripslashes($row['comment']);
        	$this->endorsement = $row['endorsement'];
        }
    }

    public function inner_html() {
        if($this->pollid == '') {
            $this->margin_action_html('suggest');
            echo "<i>No poll this week.</i>";
            return;
        }
        if (!isset($this->agreement)) {
        	$this->edit_action();
        	return;
        }
        $this->counts_html();
    }

	public function counts_html() {
        $this->margin_action_html('edit','suggest');
        $this->inner_counts_html();
    }
    
    public function inner_counts_html() {
        $this->init_counts();
        echo "<b>Q:</b> ";
        echo $this->statement;
        echo "<p/><table width=\"100%\"><tr>\n";
        for($agreement=0; $agreement <= 4; $agreement++) {
            $bin = $this->bins[$agreement];
            echo "<td align=\"center\" width=\"20%\" valign=\"bottom\">$bin</td>";
        }
        echo "<tr>\n";
        for($agreement=0; $agreement <= 4; $agreement++) {
            $count = $this->counts[$agreement];
            $value = $count > 0 ? $count : 0;
            echo "<td align=\"center\" width=\"20%\" bgcolor=\"lightgray\">$value</td>";
        }
        echo "</tr>\n<tr>
            <td colspan=\"2\">disagree</td>
	    <td align=\"center\">neutral</td>
            <td colspan=\"2\" align=\"right\">agree</td>
            </tr></table>\n";

	}

    public function edit_action() {
		if ($this->has_answered)
			$this->margin_action_html('revert');
        echo "<p><b>Q:</b> $this->statement</p>";
    	$this->html_edit_fields($this->poll_fields, "submit");

    }

	public function submit_action() {
		$errors = $this->html_save_fields($this->poll_fields);
		if ($errors) {
			$this->edit_action();
		} else {
			$this->submit();
			$this->counts_html();
		}
	}

	public function submit() {
		$agreement = array(0,1,2,3,4);
		$agreement = $agreement[$this->agreement];
		$comment = addslashes($this->comment);
		if(!$this->has_answered && !$this->has_endorsed) {
	    	$sql = "INSERT INTO poll_answer (pollid, PersonID, agreement, comment)
	    			VALUES($this->pollid, '%USER%', $agreement, '$comment')";
	    } else {
	       $sql = "UPDATE poll_answer
	        	SET agreement = $agreement,
	        	comment = '$comment'
	        	WHERE pollid = $this->pollid
	        		AND PersonID = '%USER%'";
	    }
		portal_sql($sql);
	}

	public function special_button($action) {
		echo "
			<br><input style=\"width:60px\"type=\"button\" onclick=\"javascript:function ff(o) { while((o != null) && (o.nodeName != 'FORM')) { o = o.parentNode; } return o; }
	    		portal_submit('poll.poll.1','$action',ff(this));return false;\" value=\"$action\">";
	}

	public function my_endorsements() {
		$result = portal_sql("SELECT pollid
			FROM poll_answer
			WHERE PersonID = '%USER%' AND endorsement = TRUE");
		$isEndorsed = array();
		while ($row = mysql_fetch_array($result)) {
		  $isEndorsed[$row['pollid']] = true;
		}
		return $isEndorsed;
	}

    public function suggest_action() {
        $this->margin_action_html('revert','stats');
		$this->margin_action_html('save');
		echo "<table border=0>";

		$result = portal_sql("SELECT poll_question.pollid, statement, DATE_FORMAT(post, '%M %D') as post
			FROM poll_question
			WHERE (post > " . $this->context->now() . " OR post IS NULL)");

		if (mysql_num_rows($result) > 0) {
			echo "<tr><td><td><font color=gray>Pending Questions<br>(select and save to endorse)</font>";
			$isEndorsed = $this->my_endorsements();
			while ($row = mysql_fetch_array($result)) {
				$text = stripslashes($row['statement']);
				$pollid = $row['pollid'];
				$post = $row['post'];
				$checked = isset($isEndorsed[$pollid]) ? 'checked' : '';
				$legend = $this->context->testmode ? "legend=\"$text\"" : "";
				echo "<tr><td valign=top><input type=checkbox name=$pollid $checked value=on $legend><td valign=top>$text";
				if ($post != null) {
					echo "<br><font color=gray>(scheduled for $post)</font>";
				}
			}
			echo "<tr><td>&nbsp;";
		}

		echo "<tr><td><td><font color=gray>Suggest a New Question<br>(expressed as a statement)
			<tr><td><td><textarea name=input rows=5 cols=33 wrap=virtual></textarea>
			<tr><td><td>";
	    $this->html_submit_button('save');

	    if (user_has_role($this->context, 'POLL')) {
	    	echo "<tr><td>&nbsp;<tr><td><td><font color=gray>Pollmeister Special Actions<br>(can't be undone. deselect endorsements first.)";
			$this->special_button('delete'); echo " all selected questions";
			$this->special_button('post'); echo " all selected questions";
	    }
	    echo "</table><br></form>";
    }

    public function save_action() {
		foreach ($_POST as $key => $value) {
			if (is_numeric($key) && $value == 'on') {
				$result = portal_sql("SELECT pollid FROM poll_question WHERE (post > %NOW% OR post IS NULL) AND pollid = $key");
				if (mysql_num_rows($result) > 0) {
					$result = portal_sql("SELECT pollid FROM poll_answer WHERE pollid = $key AND PersonID = '%USER%'");
					if (mysql_num_rows($result) == 0) {
						$result = portal_sql("INSERT INTO poll_answer (pollid, PersonID, endorsement) VALUES($key, '%USER%', TRUE)");
					} else {
						// $result = portal_sql("UPDATE poll_answer SET endorsement = TRUE WHERE pollid = $key AND PersonID = '%USER%'");
					}
				}
			}
		}

    	if (isset($_POST['input']) && $_POST['input'] != '') {
    		$input = addslashes($_POST['input']);
 			portal_sql("INSERT INTO poll_question (statement, submit) VALUES('$input', ". $this->context->now() .")");
    	}
    	$this->suggest_action();
    }

    public function delete_action() {
    	if (user_has_role($this->context, 'POLL')) {
			foreach ($_POST as $key => $value) {
				if (is_numeric($key) && $value == 'on') {
					portal_sql("DELETE
						FROM poll_question
						WHERE (post > " . $this->context->now() . " OR post IS NULL)
						AND (pollid = $key)");
				}
			}
    	}
    	$this->suggest_action();
    }

    public function post_action() {
    	if (user_has_role($this->context, 'POLL')) {
    		$desired = array();
			foreach ($_POST as $key => $value) {
				if (is_numeric($key) && $value == 'on') {
					$desired[] = $key;
				}
			}
	    	$weeks = it_array(portal_sql("SELECT (to_days(post) - to_days(%NOW%)) / 7 as it FROM poll_question WHERE post > %NOW%"));
	    	$endorsed = it_array(portal_sql ("SELECT q.pollid as it, COUNT(endorsement) as count FROM poll_answer as a, poll_question as q
	    		WHERE (a.pollid = q.pollid) AND (q.post IS NULL) GROUP BY a.pollid ORDER BY count DESC, q.submit"));
	    	$submitted = it_array(portal_sql ("SELECT pollid as it FROM poll_question WHERE post IS NULL ORDER BY submit"));

	    	$week = 1;
	    	foreach (array_merge($endorsed, array_diff($submitted,$endorsed)) as $pollid) {		// in priority order
	    		if (array_search ($pollid, $desired) !== false) {								// for desired ids
	    			while (array_search ($week, $weeks) !== false) {							// and available weeks
	    				$week++;
	    			}
	    			$this->schedule_week ($pollid, $week);										// schedule for that week
	    			$week++;
	    		}
	    		$desired = array_diff ($desired, array($pollid));
	    	}
	    	$this->suggest_action();
    	}
    }

    public function stats_action() {
		$this->margin_action_html('revert');
        $result = portal_sql ("SELECT statement, DATE_FORMAT(post, '%M %D') as week, COUNT(agreement) as count, agreement FROM poll_answer as a, poll_question as q
			WHERE (a.pollid = q.pollid) AND (post IS NOT NULL) AND (post <= %NOW%) GROUP BY a.pollid, a.agreement ORDER BY post DESC, agreement");
		$current = '';
		while ($row = mysql_fetch_array($result)) {
			if ($row['week'] != $current) {
				if ($current != '') print_row ($last_row, $counts);
				$current = $row['week'];
				$counts = array(0,0,0,0,0);
			}
			$counts[ceil($row['agreement'])] = $row['count'];
			$last_row = $row;
		}
		if ($current != '') print_row ($last_row, $counts);
    }



}

function it_array (&$result) {
	$rtrn = array();
	while ($row = mysql_fetch_array($result)) {
		$rtrn[] = ceil($row['it']);
	}
	return $rtrn;
}

function print_row (&$row, &$counts) {
	$total = $counts[0] + $counts[1] + $counts[2] + $counts[3] + $counts[4];
	$statement = stripslashes($row['statement']);
	print "<p><b>Q:</b> $statement<br>
		<font color=gray>($total responses in week of ${row['week']}, breakdown:
		${counts[0]}, ${counts[1]}, ${counts[2]}, ${counts[3]}, ${counts[4]} )</font></p>\n";
}


class poll_watch extends poll {
    public function id() {
        return "poll.poll_watch.1";
    }
	public function inner_html() {
        $this->margin_action_html('stats');
        if($this->pollid == '') {
            echo "<i>No poll this week.</i>";
            return;
        }
        $this->inner_counts_html();
    }	
}


?>
